*************************************************************************
*
*  PFF device, a simple device to con older programs that can't
*  print to a file
*  v. 0.03	2004 Dec 26 08:48:51	test for smsqe before increasing filename
*  v. 0.02	2004 Dec 10 19:11:45	sanity check introduced
*  v. 0.01	2004 Dec 06 17:49:24
*
*  this is the open routine.
*
*  (c) W. Lenerz 2004
*
***********************************************************************


	section device

	xdef	open
	xdef	pffname

	xref	chk_job 		; release the job
	xref	addone
	include dev1_asm_pff_includes_asm ; my own includes
	include dev8_keys_qlv		; QL vector keys
	include DEV8_keys_qdos_sms	; trap 1 keys
	include DEV8_keys_err		; error keys
	include DEV8_keys_sys
	include DEV8_keys_jcbq


;-----------------------------------------------------------------
; on open, the following registers have the following values:
; entry
; d1 = job Id (unoficial)		exit
; d3 = access key			?
; a0 = pointer to name			chan defn blk
; a3 = base of dev driver defn block	stet
; a6 = sysvars
; a7 = supervisor stack

in_use	moveq	#err.fdiu,d0
	rts
operr_out
	moveq	#err.itnf,d0		; not found
op_out	rts				; & return with error

open	bra.s	open2			; first of all check whether it's me
	nop
pffname dc.w	3,'PFF'
open2
	move.l	2(a0),d0		; first four bytes of name
	move.b	d0,d2			; keep number
	andi.l	#$dfdfdf00,d0		; upper case
	lea	usage+2(a3),a2		; point usage name
	move.l	(a2),d2 		; name
	andi.b	#0,d2			; strip off number
	cmp.l	d2,d0			; same?
	beq.s	params			;   ...yes
	lea	pffname+2,a2		;   ... no, try true name
	move.l	(a2),d2
	andi.b	#0,d2
	cmp.l	d2,d0			; same?
	bne	operr_out		; no
params	move.w	(a0),d2 		; name length
	cmp.b	#3,d2			; only 3 chars in name to open?
	beq.s	do_open 		; yes, OK
	moveq	#err.inam,d0		; now check whether params are ok
	sub.b	#'0',d2 		; port number in range 1 to 8
	blt.s	op_out			; ooops
	subq.b	#8,d2
	bgt.s	op_out			; oops again

; here we know that we must indeed open our channel
do_open
; our channel is open, now open the other one (pipe)
totregs reg	d4-d7/a2/a4-a5
lpregs	reg	a2-a4
	tst.b	isopen(a3)		; channel already open?
	bne.s	in_use			; yes, refuse to open
	move.l	a3,a2			; keep my dd defn block
	bsr	chk_job 		; is intermediary job ok and not executing?
	bne.s	in_use			; no, either we're still in use, or somebody killed that job
					;(open,immediate close,immediate open,stupid, really)
	movem.l totregs,-(a7)		; keep these regs
	tst.w	ident(a2)		; is this smsqe?
	bne.s	nosms			; no
	lea	device+10(a2),a0	; yes, increase filename
	move.l	(a0),d0
	bsr	addone
	move.l	d0,(a0)
nosms	lea	sys_iodl(a6),a4 	; a4 points to simple device driver list
	bra.s	fnd_lp			; try to find device now
fnd_dev move.l	d1,a4			; a5 = this entry in list
	lea	-$18(a4),a3		; a3 = assumed start of dev. dr. def bl.
	move.l	$20(a3),a5		; a5 = channel open routine
	lea	open,a0 		; my open routine
	cmpa.l	a0,a5			; is open routine same as mine ?
	beq.s	fnd_lp			; yes, so this is me, try again!
	movem.l lpregs,-(a7)
	lea	device(a2),a0		; a0 points to device (pipe) name
	moveq	#0,d3			; open exclusive!
	jsr	(a5)			; try to open channel
	movem.l (a7)+,lpregs
	tst.l	d0			; open ok ?
	beq.s	fnd_ok			;  ... yes, so go on
	cmpi.l	#err.itnf,d0		;  ... no, was it err_nf?
	bne.s	op_err			;     ... no, another error, so leave
fnd_lp	move.l	(a4),d1 		; is there one more device?
	bne.s	fnd_dev 		;  ... yes
	moveq	#err.itnf,d0		;  ... no
	bra.s	op_err			;  if we get here, we couldn't find the device
fnd_ok
	st	isopen(a2)		; flag that channel is open
	move.l	jctblk(a2),a4		; pointer to job control block
	move.w	#8,jcb_pacc(a4) 	; accumulated prio = 0,increase=8 ...
	move.l	#waterm1,jcb_d1(a4)	; =  release intermediary job
	move.l	#waterm2,jcb_d2(a4)	; set up some regs in the intermediary ...
	move.l	#waterm3,jcb_d3(a4)	; job, to allow sanity check
	move.l	a0,jcb_a1(a4)		; point chan defn block in A0
	lea	device(a2),a5		; device to open
	tst.w	ident(a2)		; are we smsqe?
	bne.s	smsok			; yes, use this name
	lea	qdosp(a2),a5		; point to fake qdos pipe name
smsok	move.l	a5,jcb_a0(a4)		; set pointer to device name for intermediary job
	move.l	a3,newdd(a2)		; dd defn block for other channel
	move.l	$1c(a3),ciorout(a2)	; keep io routine for that driver
	move.l	$24(a3),cclose(a2)	; and close routine
	move.l	a0,chandb(a3)		; keep ptr to channel defn block
op_exit movem.l (a7)+,totregs		; now get my regs back
	clr.l	d0
	rts

op_err	move.l	a2,a3
	movem.l (a7)+,totregs		; now get my regs back
	rts
	end



	 end
